from .common import BaseRequest, ProviderKind, Settings, SuccessResponse
from .fallback import AgentFallbackRequest, AudioFallbackRequest, TranscribeFallbackRequest, FallbackStrategy
from .agent.agent import AgentRequest, AgentResponse, AgentResponseUsage, ContextOverflowStrategy
from .agent.chat import Message, MessageRole, TextMessage, BinaryMessage, ToolCallMessage, ToolReturnMessage
from .agent.tools import Tool, ToolInput, ToolKind, WebSearchInput, MCPStreamableServerInput
from .agent.output_schema import OutputSchema, PropertyDef
from .audio.speak import SpeakMimeType, SpeakRequest, SpeakStreamResponse
from .audio.transcribe import TranscribeRequest, TranscribeResponse, File
from .transcription import TranscriptionInitWsRequest, TranscriptionAudioChunkWsRequest, TranscriptionWsResponse


__all__ = [
    # Common
    "BaseRequest",
    "ProviderKind",
    "Settings",
    "SuccessResponse",
    # Fallback
    "AgentFallbackRequest",
    "AudioFallbackRequest",
    "TranscribeFallbackRequest",
    "FallbackStrategy",
    # Agent
    "AgentRequest",
    "AgentResponse",
    "AgentResponseUsage",
    "ContextOverflowStrategy",
    "Message",
    "MessageRole",
    "TextMessage",
    "BinaryMessage",
    "ToolCallMessage",
    "ToolReturnMessage",
    "Tool",
    "ToolInput",
    "ToolKind",
    "WebSearchInput",
    "MCPStreamableServerInput",
    "OutputSchema",
    "PropertyDef",
    # Audio
    "SpeakMimeType",
    "SpeakRequest",
    "SpeakStreamResponse",
    "TranscribeRequest",
    "TranscribeResponse",
    "File",
    # Real-time Transcription
    "TranscriptionInitWsRequest",
    "TranscriptionAudioChunkWsRequest",
    "TranscriptionWsResponse",
]


from .agent import AgentRequest, AgentResponse, AgentResponseUsage, ContextOverflowStrategy
from .chat import Message, MessageRole, TextMessage, BinaryMessage, ToolCallMessage, ToolReturnMessage
from .tools import Tool, ToolInput, ToolKind, WebSearchInput, MCPStreamableServerInput
from .output_schema import OutputSchema, PropertyDef


__all__ = [
    "AgentRequest",
    "AgentResponse",
    "AgentResponseUsage",
    "ContextOverflowStrategy",
    "Message",
    "MessageRole",
    "TextMessage",
    "BinaryMessage",
    "ToolCallMessage",
    "ToolReturnMessage",
    "Tool",
    "ToolInput",
    "ToolKind",
    "WebSearchInput",
    "MCPStreamableServerInput",
    "OutputSchema",
    "PropertyDef",
]


# models for chat messages
from pydantic import BaseModel, Field, model_validator, field_serializer
from enum import Enum
from typing import Optional, Union, Any, Dict

class MessageRole(str, Enum):
    USER = "user"
    MODEL = "model"
    SYSTEM = "system"
    TOOL_CALL = "tool_call"
    TOOL_RETURN = "tool_return"


class Message(BaseModel):
    role: Union[MessageRole, str] = Field(..., description="The role of the message")
    
class TextMessage(Message):
    content: str = Field(..., description="The content of the message")

class BinaryMessage(Message):
    """always from user"""
    content: str = Field(..., description="The base64 encoded content of the message")
    mime_type: str = Field(..., description="The MIME type of the content, only user can supply such")
    caption: Optional[str] = Field(None, description="Caption for the binary message")

    @model_validator(mode="after")
    def validate_content(self) -> "BinaryMessage":
        if self.role == MessageRole.MODEL:
            raise ValueError("MIME type are meant for user messages only")
        return self

class ToolCallMessage(Message):
    """Message representing a tool call made by the agent"""
    tool_name: str = Field(..., description="The name of the tool being called")
    args: Union[Dict[str, Any], str] = Field(..., description="The arguments passed to the tool")

class ToolReturnMessage(Message):
    """Message representing the return value from a tool call"""
    tool_name: str = Field(..., description="The name of the tool that was called")
    content: Any = Field(..., description="The return value from the tool")

